$title	A Highly Simplified Climate Model

SET	t		Time periods /1*40/;

parameter
        m0              CO2-equiv concent. 1965 billion tons carbon     /677/,
        tl0             Lower stratum temperature (C) 1965              /.10/,
        t0              Atmospheric temperature (C) 1965                /.2/,
        atret           Marginal atmospheric retention rate             /.64/,
        c1              Coefficient for upper level                     /.226/,
        lam             Climate feedback factor                         /1.41/,
        c3              Coefficient trans upper to lower stratum        /.440/,
        c4              Coeff of transfer for lower level               /.02/,

        r               Rate of social time preference per year         /.03/,
        gl0             Growth rate of population per decade            /.223/,
        dlab            Decline rate of population growth per dec       /.195/,
        deltam          Removal rate carbon per decade                  /.0833/,
        ga0             Initial growth rate for technology per decade   /.15/,
        dela            Decline rate of technology per decade           /.11/,
        gsigma          Growth of sigma per decade                      /-.1168/,
        sig0            CO2-equiv-GWP ratio                             /.519/,
        sigma(t)        Emissions-output ratio,
        L0              1965 world population millions                  /3369/,
        k0              1965 value capital billions 1989 US dollars     /16.03/,
        gamma           Capital elasticity in output                    /.25/,

        a0              Initial level of total factor productivity      /.00963/,
        L(t)            Level of population and labor,
        al(t)           Level of total factor productivity (TFP),
        ga(t)           Growth rate of TFP from 0 to T,
        gl(t)           Growth rate of labor 0 to T,
        gsig(t)         Cumulative improvement of energy efficiency
	ebau(t)		Baseline emissions;

gsig(t) = (gsigma/dela)*(1-EXP(-dela*(ORD(t)-1)));
sigma(t)=sig0*EXP(gsig(t));
gl(t) = (gl0/dlab)*(1-EXP(-dlab*(ORD(t)-1)));
L(t)=L0*EXP(gl(t))*.9;
ga(t)= (ga0/dela)*(1-EXP(-dela*(ORD(t)-1)));
al(t) =a0*EXP(ga(t));

ebau(t) = 10 * sigma(t) * al(t) * (k0*L(t)/L0)**gamma * L(t)**(1-gamma);

set	t200(t)		/1*20/,
	tlbl(t)		/5 2050, 10 2100, 15 2150, 20 2200/;

$setglobal domain t200
$setglobal labels tlbl
$setglobal gp_opt0 set ylabel 'tons CO2'
$setglobal gp_opt1 set xlabel 'year'
$batinclude plot ebau
$setglobal gp_opt0 

parameter
        m(t)            CO2-equiv concentration billion t
        forc(t)         Radiative forcing - W per m2
        forcoth(t)      Exogenous forcings from other greenhouse gases,
        te(t)           Temperature - atmosphere C
	teref(t)	Reference temperature path
        tl(t)           Temperature - lower ocean C
        termv           Terminal value of atmophere
        deltaE          Difference iterval /0.001/;

set	tfirst(t)	The first time period;

tfirst(t) = yes$(ord(t)=1);

*	Initial conditions for climate model:
m(tfirst) = m0;  te(tfirst) = t0;  tl(tfirst) = tl0;  forcoth(t) = 1.42;

parameter	climate		Climate evolution;
parameter	eref(t)		Reference emissions
		E(t)		Currently estimated emissions path,
		grad		Temperature gradient,
		teinit		Initial temperature path;

*	Save the baseline emissions path:

E(t) = ebau(t);

*	Write out a "subroutine file" for computing the climate model:

$onecho >climatemodel.gms
loop(t,
	m(t)		= 590 + atret*eref(t) + (1-deltam)*(m(t-1)-590) + m0$tfirst(t);
	forc(t)		= 4.1*(LOG(m(t)/590)/LOG(2)) + forcoth(t);
	te(t)		= te(t-1)+c1*(forc(t-1)-lam*te(t-1)-c3*(te(t-1)-tl(t-1))) + t0$tfirst(t);
	tl(t)		= tl(t-1)+c4*(te(t-1)-tl(t-1)) + tl0$tfirst(t);
	teref(t)	= te(t);
);
$offecho

*	Write a second subroutine to compute the gradients:

alias (t,tp);

$onecho >jacobian.gms
	eref(t) = E(t);
$include climatemodel
	teinit(t) = teref(t); 
	grad(t,tp) = 0;	
	loop(tp,eref(tp) = eref(tp) + deltaE;
$include climatemodel
	  grad(t,tp)= (teref(t)-teinit(t)) / deltaE;
	  eref(tp)  = eref(tp) - deltaE;);
	  teref(t)  = teinit(t);

$offecho


parameter	climate		Temperature and abatement policy paths;

*	Compute the climate path and its Jacobian:

$include jacobian

climate(t,"te0") = te(t);

*	Plot some dependencies:

parameter	gradients	Marginal contributions of selected year emissions;
gradients(t,"2050") = grad("5",t);
gradients(t,"2100") = grad("10",t);
gradients(t,"2150") = grad("15",t);
gradients(t,"2200") = grad("20",t);
$batinclude plot gradients


parameter		pv(t)		Present value cost;

pv(t) = 1/(1+r)**(ord(t)-1);

variables	OBJ		Objective function
		ABATE(t)	Abatement measures
		ABATELVL	Abatement level (uniform);

positive variable abate;

equations	objdef, avetemp, ratelimit, unifabate;

objdef..		obj =e= sum(t, pv(t) * ABATE(t));
unifabate(t)..		ABATE(t) =e= ABATELVL * Ebau(t);
ratelimit(t+1)..	ABATE(t+1) =l= 1.2 * ABATE(t) + 0.10*Ebau(t);
avetemp(t)..		teref(t) + sum(tp, grad(t,tp)*(Ebau(tp)-ABATE(tp)-eref(tp))) =l= 2;

model optabate /objdef, avetemp, ratelimit/;

*	Solve iteration 1:

solve optabate using nlp minimizing obj;
climate(t,"a1") = ABATE.L(t)/ebau(t);
E(t) = ebau(t) - ABATE.L(t);
$include jacobian
climate(t,"te1") = te(t);

*	Solve iteration 2

solve optabate using nlp minimizing obj;
climate(t,"a2") = ABATE.L(t)/ebau(t);
E(t) = ebau(t) - ABATE.L(t);
$include jacobian
climate(t,"te2") = te(t);

*	Solve iteration 3

solve optabate using nlp minimizing obj;
climate(t,"a3") = ABATE.L(t)/ebau(t);
E(t) = ebau(t) - ABATE.L(t);
$include jacobian
climate(t,"te3") = te(t);
parameter	compare	Comparison of optimal and uniform abatement;
compare(t,"A-Opt") = ABATE.L(t)/ebau(t);
compare(t,"T-Opt") = te(t);

model abateunif /objdef, avetemp, unifabate/;

solve abateunif using nlp minimizing obj;
climate(t,"u1") = ABATE.L(t)/ebau(t);
E(t) = ebau(t) - ABATE.L(t);
$include jacobian
climate(t,"u1") = te(t);

solve abateunif using nlp minimizing obj;
climate(t,"u2") = ABATE.L(t)/ebau(t);
E(t) = ebau(t) - ABATE.L(t);
$include jacobian
climate(t,"u2") = te(t);

compare(t,"A-Unif") = ABATE.L(t)/ebau(t);
compare(t,"T-Unif") = te(t);

display climate;

*	Plot the results:

$batinclude plot climate
$batinclude plot compare
